import os

from rosdistro import get_index, get_release_cache

FILES_DIR = os.path.normpath(os.path.join(os.path.dirname(__file__), 'files'))


def test_get_release_cache():
    url = 'file://' + FILES_DIR + '/index.yaml'
    i = get_index(url)
    get_release_cache(i, 'foo')



import os
import yaml

from rosdistro import get_index, get_release, get_release_build_files, get_release_builds
from rosdistro.build_file import BuildFile
from rosdistro.loader import load_url

FILES_DIR = os.path.normpath(os.path.join(os.path.dirname(__file__), 'files'))


def test_build_file():
    url = 'file://' + FILES_DIR + '/foo-build.yaml'
    yaml_str = load_url(url)
    data = yaml.load(yaml_str)
    BuildFile('foo', data)


def test_get_release_build_files():
    url = 'file://' + FILES_DIR + '/index.yaml'
    i = get_index(url)
    get_release_build_files(i, 'foo')


def test_get_release_builds():
    url = 'file://' + FILES_DIR + '/index.yaml'
    i = get_index(url)
    d = get_release(i, 'foo')
    builds = get_release_builds(i, d)
    assert len(builds) == 1



# Software License Agreement (BSD License)
#
# Copyright (c) 2013, Open Source Robotics Foundation, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Open Source Robotics Foundation, Inc. nor
#    the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from catkin_pkg.package import parse_package_string


class DependencyWalker(object):

    ALL_DEPENDENCY_TYPES = ['buildtool', 'build', 'run', 'test']

    DIRECT_DEPENDS = {'buildtool': [], 'build': [], 'run': [], 'test': []}

    FULL_DEPENDS = {
        'buildtool': ALL_DEPENDENCY_TYPES,
        'build': ALL_DEPENDENCY_TYPES,
        'run': ALL_DEPENDENCY_TYPES,
        'test': ALL_DEPENDENCY_TYPES
    }

    def __init__(self, release_instance):
        self._release_instance = release_instance
        self._packages = {}
        self._dependency_cache = {
            'buildtool': {},
            'build': {},
            'run': {},
            'test': {}
        }

    def _get_package(self, pkg_name):
        if pkg_name not in self._packages:
            pkg_xml = self._release_instance.get_package_xml(pkg_name)
            pkg = parse_package_string(pkg_xml)
            self._packages[pkg_name] = pkg
        return self._packages[pkg_name]

    def get_depends(self, pkg_name, depend_type, ros_packages_only=False):
        '''Return the set of package dependencies of the specified type.'''
        deps = self._get_dependencies(pkg_name, depend_type)
        if ros_packages_only:
            deps = deps & set(self._release_instance.packages.keys())
        return deps

    def get_recursive_depends(self, pkg_name, depend_type, ros_packages_only=False):
        '''Return the set of recursive package dependencies of the specified type.'''
        if pkg_name not in self._dependency_cache[depend_type]:
            deps = self._get_dependencies(pkg_name, depend_type)
            for dep in sorted(deps):
                if dep not in self._release_instance.packages.keys():
                    continue
                rec_depends = self.get_recursive_depends(dep, depend_type)
                deps = deps | rec_depends
            self._dependency_cache[depend_type][pkg_name] = deps

        depends = self._dependency_cache[depend_type][pkg_name]
        if ros_packages_only:
            depends = depends & set(self._release_instance.packages.keys())
        return depends

    def get_recursive_depends2(self, pkg_name, depend_types, ros_packages_only=False):
        '''Return the set of recursive package dependencies of the specified types.'''
        depends = set([])
        for dep_type in sorted(depend_types):
            deps = self.get_recursive_depends(pkg_name, dep_type, ros_packages_only=ros_packages_only)
            depends = depends | deps
        return depends

    def get_mapped_depends(self, pkg_name, depend_walk, ros_packages_only=False):
        '''Return a dict mapping from dependency types to sets of (recursive) package dependencies specified by the walk-map.'''
        depends = {}
        for dep_type in sorted(depend_walk.keys()):
            deps = self._get_dependencies(pkg_name, dep_type)
            rec_deps = set([])
            for dep in sorted(deps):
                if dep not in self._release_instance.packages.keys():
                    continue
                x = self.get_recursive_depends2(dep, depend_walk[dep_type], ros_packages_only=ros_packages_only)
                rec_deps |= x
            if ros_packages_only:
                deps = deps & set(self._release_instance.packages.keys())
            depends[dep_type] = deps | rec_deps
        return depends

    def _get_dependencies(self, pkg_name, dep_type):
        assert dep_type in DependencyWalker.ALL_DEPENDENCY_TYPES
        pkg = self._get_package(pkg_name)
        deps = {
            'buildtool': pkg.buildtool_depends,
            'build': pkg.build_depends,
            'run': pkg.run_depends,
            'test': pkg.test_depends
        }
        return set([d.name for d in deps[dep_type]])



# Software License Agreement (BSD License)
#
# Copyright (c) 2013, Open Source Robotics Foundation, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Open Source Robotics Foundation, Inc. nor
#    the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from .test_file import TestFile


class ReleaseFile(TestFile):

    def __init__(self, name, data):
        super(ReleaseFile, self).__init__(name, data, expected_type='release')

        for repo in self.repositories.values():
            assert repo.type == 'git'
            if repo.version is not None:
                assert 'release' in repo.tags

        self.platforms = []
        if 'platforms' in data:
            for platform_name in data['platforms']:
                platform_name = str(platform_name)
                assert platform_name not in self.platforms
                self.platforms.append(platform_name)

    def get_data(self):
        data = super(ReleaseFile, self).get_data()
        if self._type == 'release':
            data['gbp-repos'] = {'You must update to a newer rosdep version by calling..sudo apt-get update && sudo apt-get install python-rosdep (make sure to uninstall the pip version on Ubuntu': None}
        data['platforms'] = self.platforms
        return data



# Software License Agreement (BSD License)
#
# Copyright (c) 2013, Open Source Robotics Foundation, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Open Source Robotics Foundation, Inc. nor
#    the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.


class BuildFile(object):

    def __init__(self, name, data):
        assert data['type'] == 'build'
        self.version = int(data['version'])
        assert self.version == 1

        self.name = name

        self.package_whitelist = []
        if 'package_whitelist' in data:
            self.package_whitelist = data['package_whitelist']
            assert isinstance(self.package_whitelist, list)
        self.package_blacklist = []
        if 'package_blacklist' in data:
            self.package_blacklist = data['package_blacklist']
            assert isinstance(self.package_blacklist, list)

        self.notify_emails = []
        self.notify_maintainers = False
        self.notift_committers = False
        if 'notifications' in data:
            if 'emails' in data['notifications']:
                self.notify_emails = data['notifications']['emails']
                assert isinstance(self.notify_emails, list)
            if 'maintainers' in data['notifications'] and data['notifications']['maintainers']:
                self.notify_maintainers = True
            if 'committers' in data['notifications'] and data['notifications']['committers']:
                self.notift_committers = True

        assert 'targets' in data
        assert data['targets']
        self.targets = {}
        for platform_name in data['targets']:
            platform_name = str(platform_name)
            assert platform_name not in self.targets
            self.targets[platform_name] = []
            for arch_name in data['targets'][platform_name]:
                self.targets[platform_name].append(arch_name)

        assert 'jenkins_url' in data
        self.jenkins_url = str(data['jenkins_url'])
        self.apt_mirrors = []
        if 'apt_mirrors' in data:
            self.apt_mirrors = data['apt_mirrors']
            assert isinstance(self.apt_mirrors, list)
        self.apt_target_repository = None
        if 'apt_target_repository' in data:
            self.apt_target_repository = str(data['apt_target_repository'])
            if not self.apt_mirrors:
                self.apt_mirrors.append(self.apt_target_repository)
        assert self.apt_mirrors

        self.sync_package_count = None
        self.sync_packages = []
        if 'sync' in data:
            if 'package_count' in data['sync']:
                self.sync_package_count = int(data['sync']['package_count'])
            if 'packages' in data['sync']:
                self.notify_maintainers = data['sync']['packages']
                assert isinstance(self.sync_packages, list)

    def get_data(self):
        data = {}
        data['type'] = 'build'
        data['version'] = 1
        if self.package_whitelist:
            assert isinstance(self.notify_emails, list)
            data['package_whitelist'] = self.package_whitelist
        data['package_blacklist'] = self.package_blacklist

        if self.notify_emails or self.notify_maintainers or self.notift_committers:
            data['notifications'] = {}
            if self.notify_emails:
                assert isinstance(self.notify_emails, list)
                data['notifications']['emails'] = self.notify_emails
            if self.notify_maintainers is not None:
                data['notifications']['maintainers'] = bool(self.notify_maintainers)
            if self.notift_committers is not None:
                data['notifications']['committers'] = bool(self.notift_committers)

        data['targets'] = {}
        for platform_name in self.targets:
            assert isinstance(self.targets[platform_name], list)
            data['targets'][platform_name] = self.targets[platform_name]

        data['jenkins_url'] = self.jenkins_url
        if self.apt_mirrors:
            data['apt_mirrors'] = []
            if len(self.apt_mirrors) != 1 or self.apt_mirrors[0] != self.apt_target_repository:
                for mirror in self.apt_mirrors:
                    data['apt_mirrors'].append(mirror)
        if self.apt_target_repository:
            data['apt_target_repository'] = self.apt_target_repository

        return data



import os

from rosdistro import get_index

FILES_DIR = os.path.normpath(os.path.join(os.path.dirname(__file__), 'files'))


def test_get_index():
    url = 'file://' + FILES_DIR + '/index.yaml'
    get_index(url)



# Software License Agreement (BSD License)
#
# Copyright (c) 2013, Open Source Robotics Foundation, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Open Source Robotics Foundation, Inc. nor
#    the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from .repository import Repository


class DocFile(object):

    def __init__(self, name, data, expected_type=None):
        expected_type = expected_type if expected_type is not None else 'doc'
        assert data['type'] == str(expected_type)
        self._type = str(expected_type)
        self.version = int(data['version'])
        assert self.version == 1

        self.name = name

        self.repositories = {}
        if 'repositories' in data:
            for repo_name in data['repositories']:
                repo_data = data['repositories'][repo_name]
                repo = Repository(repo_name, repo_data)
                # TODO use smaller data structure
                repo.tags = None
                repo.status = None
                repo.status_description = None
                repo.package_names = None

                self.repositories[repo_name] = repo

    def get_data(self):
        data = {}
        data['type'] = self._type
        data['version'] = 1
        data['repositories'] = {}
        for repo_name in sorted(self.repositories):
            repo = self.repositories[repo_name]
            data['repositories'][repo_name] = repo.get_data()
        return data



# Software License Agreement (BSD License)
#
# Copyright (c) 2013, Open Source Robotics Foundation, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Open Source Robotics Foundation, Inc. nor
#    the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# legacy imports
import common
from rosdistro import walks
from rosdistro import RosDistro
from develdistro import DevelDistro
from aptdistro import AptDistro

import logging
import os
import yaml

logger = logging.getLogger('rosdistro')

from build import Build
from build_file import BuildFile
from doc_file import DocFile
from index import Index
from loader import load_url
from manifest_provider.cache import CachedManifestProvider
from release import Release
from release_cache import ReleaseCache
from release_file import ReleaseFile
from test_file import TestFile

DEFAULT_INDEX_URL = 'https://raw.github.com/ros/rosdistro/rep137/releases/index.yaml'


def get_index_url():
    if 'ROSDISTRO_INDEX_URL' in os.environ:
        return os.environ['ROSDISTRO_INDEX_URL']
    return DEFAULT_INDEX_URL


def get_index(url):
    logger.debug('Load index from "%s"' % url)
    yaml_str = load_url(url)
    data = yaml.load(yaml_str)
    base_url = os.path.dirname(url)
    return Index(data, base_url)


def get_cached_release(index, dist_name, cache=None, allow_lazy_load=False):
    if cache is None:
        try:
            cache = get_release_cache(index, dist_name)
        except Exception:
            if not allow_lazy_load:
                raise
            # create empty cache instance
            rel_file_data = get_release_file_data(index, dist_name, 'release')
            cache = ReleaseCache(dist_name, rel_file_data=rel_file_data)
    rel = Release(
        cache.release_file,
        [CachedManifestProvider(cache, Release.default_manifest_providers if allow_lazy_load else None)])
    assert cache.release_file.name == dist_name
    return rel


def get_release(index, dist_name):
    rel_file = get_release_file(index, dist_name)
    return Release(rel_file)


def get_release_file(index, dist_name):
    data = get_release_file_data(index, dist_name, 'release')
    return ReleaseFile(dist_name, data)


def get_release_cache(index, dist_name):
    if dist_name not in index.distributions.keys():
        raise RuntimeError("Unknown release: '{0}'".format(dist_name))
    dist = index.distributions[dist_name]
    if 'release_cache' not in dist.keys():
        raise RuntimeError("Release has no cache: '{0}'".format(dist_name))
    url = dist['release_cache']

    logger.debug('Load cache from "%s"' % url)
    yaml_str = load_url(url)
    data = yaml.load(yaml_str)
    return ReleaseCache(dist_name, data)


def get_test_file(index, dist_name):
    data = get_release_file_data(index, dist_name, 'test')
    return TestFile(dist_name, data)


def get_release_build_files(index, dist_name):
    return _get_build_files(index, dist_name, 'release_build')


def get_test_build_files(index, dist_name):
    return _get_build_files(index, dist_name, 'test_build')


def _get_build_files(index, dist_name, type_):
    data = get_release_file_data(index, dist_name, type_)
    build_files = []
    for d in data:
        build_files.append(BuildFile(dist_name, d))
    return build_files


def get_release_builds(index, dist):
    build_files = get_release_build_files(index, dist.name)
    return _get_builds(dist, build_files)


def get_test_builds(index, dist):
    build_files = get_test_build_files(index, dist.name)
    return _get_builds(dist, build_files)


def _get_builds(dist, build_files):
    builds = []
    for build_file in build_files:
        builds.append(Build(dist, build_file))
    return builds


def get_release_file_data(index, dist_name, type_):
    if dist_name not in index.distributions.keys():
        raise RuntimeError("Unknown release: '{0}'".format(dist_name))
    dist = index.distributions[dist_name]
    if type_ not in dist.keys():
        raise RuntimeError('unknown release type "%s"' % type_)
    url = dist[type_]

    def _load_yaml_data(url):
        logger.debug('Load file from "%s"' % url)
        yaml_str = load_url(url)
        return yaml.load(yaml_str)

    if not isinstance(url, list):
        data = _load_yaml_data(url)
    else:
        data = []
        for u in url:
            data.append(_load_yaml_data(u))
    return data


def get_doc_file(index, dist_name):
    data = get_release_file_data(index, dist_name, 'doc')
    return DocFile(dist_name, data)



import os
import yaml

from rosdistro import get_index, get_release_file
from rosdistro.release_file import ReleaseFile
from rosdistro.loader import load_url

FILES_DIR = os.path.normpath(os.path.join(os.path.dirname(__file__), 'files'))


def test_release_file():
    url = 'file://' + FILES_DIR + '/foo.yaml'
    yaml_str = load_url(url)
    data = yaml.load(yaml_str)
    ReleaseFile('foo', data)


def test_get_release_file():
    url = 'file://' + FILES_DIR + '/index.yaml'
    i = get_index(url)
    rel_file = get_release_file(i, 'foo')

    assert('bar_repo' in rel_file.repositories)
    repo = rel_file.repositories['bar_repo']
    assert repo.package_names == ['bar_repo']



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='rosdistro',
    version='0.1.17',
    packages=[
        'rosdistro',
        'rosdistro.manifest_provider'
    ],
    package_dir = {
        'rosdistro': 'src/rosdistro',
        'rosdistro.manifest_provider': 'src/rosdistro/manifest_provider'
    },
    scripts = [
        'scripts/rosdistro',
        'scripts/rosdistro_build_cache',
        'scripts/rosdistro_convert',
        'scripts/rosdistro_generate_cache'
    ],
    install_requires=['empy', 'PyYAML', 'argparse', 'rospkg', 'distribute'],
    package_data = {'rosdistro': ['resources/templates/*']},
    author='Wim Meeussen',
    author_email='wim@hidof.com',
    maintainer='Dirk Thomas',
    maintainer_email='dthomas@osrfoundation.org',
    url='http://www.ros.org/wiki/rosdistro',
    download_url='http://pr.willowgarage.com/downloads/rosdistro/',
    keywords=['ROS'],
    classifiers=['Programming Language :: Python',
                 'License :: OSI Approved :: BSD License'],
    description="A tool to work with rosdistro files",
    long_description="""A tool to work with rosdistro files""",
    license='BSD'
)



# Software License Agreement (BSD License)
#
# Copyright (c) 2013, Open Source Robotics Foundation, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Open Source Robotics Foundation, Inc. nor
#    the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import difflib
import yaml

from . import get_index, get_release_file, get_test_file
from .loader import load_url


def verify_files_parsable(index_url):
    index = get_index(index_url)
    for dist_name in sorted(index.distributions):
        if dist_name not in ['groovy']:
            continue
        dist = index.distributions[dist_name]
        if 'release' in dist:
            url = dist['release']
            yaml_str = load_url(url)
            dist_file = get_release_file(index, dist_name)
            if yaml_str != _to_yaml(dist_file.get_data()):
                diff = difflib.unified_diff(
                    yaml_str.splitlines(), _to_yaml(dist_file.get_data()).splitlines(),
                    'release.org', 'release.load-and-save',
                    n=0, lineterm='')
                for line in diff:
                    print(line)
                assert False
        if 'test' in dist:
            url = dist['test']
            yaml_str = load_url(url)
            dist_file = get_test_file(index, dist_name)
            if yaml_str != _to_yaml(dist_file.get_data()):
                diff = difflib.unified_diff(
                    yaml_str.splitlines(), _to_yaml(dist_file.get_data()).splitlines(),
                    'test.org', 'test.load-and-save',
                    n=0, lineterm='')
                for line in diff:
                    print(line)
                assert False


def _to_yaml(data):
    yaml_str = yaml.dump(data, default_flow_style=False)
    yaml_str = yaml_str.replace(': null', ':')
    yaml_str = yaml_str.replace(': {}', ':')
    return yaml_str
